; Eastweed - EAsy STatic WEb EDitor
; Copyright (C) 2006, 2007, 2008, 2009 Gianni Ciolli <gc@inatna.it> 
; file module-text2schtml.scm, version 0.7beta, date 2009-02-26
;
; Based on existing code (C) 2001, 2004 by Marco Maggesi
; <maggesi@math.unifi.it> and Gianni Ciolli <ciolli@math.unifi.it>
;
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

;; module-text2schtml.scm - this Eastweed module implements a highly
;; simplified parser which produces schtml code from suitable text
;; files.

;;; TODO: the directory for input verbatim files should be a subdir of
;;; the present directory, in order to comply with .htaccess
;;; directives; thus the tokens->schmtl.struct procedure should know
;;; the directory in which the currently processed file resides.

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (blank? c)
  (case c
    ((#\space #\newline #\tab) #t)
    (else #f)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (comment? c)
  (equal? c #\#))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (special? c)
  (case c
    ((#\space #\newline #\tab #\[ #\]) #t)
    (else #f)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (newline? c)
  (equal? c #\newline))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (space? c)
  (equal? c #\space))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (delimiter? c)
  (case c
    ((#\[ #\]) #t)
    (else #f)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (open-delimiter? c)
  (equal? c #\[))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (closed-delimiter? c)
  (equal? c #\]))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (howmany-newlines l) ;; counts newlines in list
  (let loop ((l l)
	     (n 0))
    (if (null? l)
	n
	(loop (cdr l)
	      (if (newline? (car l))
		  (+ 1 n)
		  n)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (remove-comments-and-reverse l0)
  (let loop ((l0 l0)
	     (a '())
	     (x #f)) ;; whether current character is commented
    (if (null? l0)
	a
	(let ((c (car l0))
	      (l (cdr l0)))
	  (if x
	      (loop l a (not (newline? c)))
	      (if (comment? c)
		  (loop l a #t)
		  (loop l (cons c a) #f)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (file->charlist file)
  (call-with-input-file 
      file
    (lambda (port)
      (let loop ((a '()))
	(let ((c (read-char port)))
	  (if (eof-object? c)
	      (reverse a)
	      (loop (cons c a))))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (remove-trailing-blanks l)
  (cond
   ((null? l)
    l)
   ((blank? (car l))
    (remove-trailing-blanks (cdr l)))
   (else
    l)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (compact-blanks l)
  (let loop ((l l)
	     (b '()) ;; blanks
	     (a '()))
    (cond
     ;; EOF
     ((null? l) 
      (reverse a))
     ;; blank
     ((and (blank? (car l)))
      (loop (cdr l)
	    (cons (car l) b)
	    a))
     ;; first non-blank
     ((not (null? b))
      (loop l
	    '()
	    (if (> (howmany-newlines b) 1)
		(cons #\newline a)
		(cons #\space a))))
     ;; non-first non-blank
     (else
      (loop (cdr l)
	    '()
	    (cons (car l) a))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (build-structure l0)
  (letrec ((f (lambda (l)
		;; return two values l and r, where l is the tokenized
		;; list and r is the remaining sublist. r can be
		;; either #f or a list; in the former case the
		;; procedure terminated because the input list ended,
		;; in the latter case because 'closed-delimiter was
		;; found.
		(let loop ((l l)
			   (a '()))
		  (cond
		   ((null? l)
		    (values (reverse a) #f))
		   ((equal? 'closed-delimiter (car l))
		    (values (reverse a) (cdr l)))
		   ((equal? 'open-delimiter (car l))
		    (let-values 
		     (((pl r) (f (cdr l))))
		     (if r
			 (loop r (cons pl a))
			 (errmsg "missing a closing delimiter; last characters are " pl))))
		   (else
		    (loop (cdr l)
			  (cons (car l) a))))))))
    (let-values
     (((pl r) (f l0)))
     (if r
	 (errmsg "exceeding closing delimiters; following characters are " r)
	 pl))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (recursive-subst what with)
  (lambda (l)
    (let loop ((l l)
	       (a '()))
      (if (null? l)
	  (reverse a)
	  (loop (cdr l)
		(cons (cond
		       ((pair? (car l))
			((recursive-subst what with) (car l)))
		       ((equal? (car l) what)
			with)
		       (else
			(car l)))
		      a))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (inline-tag? t)
  (case t
    ((link link-hash email file u i b tt image-inline) #t)
    (else #f)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (replace-newlines.struct l)
  ;; This procedure consumes a structured list and returns a
  ;; structured list after killing top-level newlines and replacing
  ;; other newlines with (br).
  (let loop ((l l)
	     (a '()))
    (if (null? l)
	(reverse a)
	(if (newline? (car l))
	    (loop (cdr l) a)
	    (loop (cdr l) (cons
			   ((recursive-subst #\space " ")
			    ((recursive-subst #\newline '(br))
			     (car l)))
			   a))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define replace-spaces.struct
  ;; This procedure consumes a structured list and returns a
  ;; structured list after killing top-level newlines, replacing
  ;; other newlines with (br), and replacing #\space with " "
  (recursive-subst #\space " "))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (make-paragraphs.struct l)
  ;; This procedure consumes a structured list (i.e. a nested
  ;; charlist) and returns a structured list where paragraphs are
  ;; split. Example: if the top list is
  ;;
  ;; x1 ... x2 y1 x3 ... x4 z2 ... y3 x5 ... x6 
  ;;
  ;; where the xi are strings, the yi are lists beginning with an
  ;; inline tag, and the zi are either newlines or lists beginning
  ;; with a paragraph-breaking tag, then the output is
  ;;
  ;; (p x1 ... x2 y1 x3 ... x4) z2  ... z3 (p x5 ... x6)
  ;;
  ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
  (let loop ((l l)
	     (a '())
	     (t '()))
    ;(deb "----------------------------------------------------------------------" #f)
    ;(deb "mp.s:l" l)
    ;(deb "mp.s:a" a)
    ;(deb "mp.s:t" t)
    (cond
     ;; (1) Did the input list end?
     ((null? l)
      (if (null? t)
	  (reverse a)
	  (loop l (cons (cons 'p (reverse t)) a) '())))
     ;; (2) Is (car l) a newline?
     ((newline? (car l))
      (loop (cdr l)
	    (cons (car l) 
		  (if (null? t) a (cons (cons 'p (reverse t)) a)))
	    '()))
     ;; (3) Is (car l) a paragraph-breaking sublist?
     ((and (list? (car l))
	   (not (inline-tag? (caar l))))
      (loop (cdr l)
	    (cons (car l) 
		  (if (null? t) a (cons (reverse t) a)))
	    '()))
     ;; (4) (car l) does not break the current paragraph
     (else
      (loop (cdr l)
	    a
	    (cons (car l) t))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (tokenize.struct sl)
  ;; this procedure consumes a structured list (i.e. a nested
  ;; charlist) and returns a structured list where characters are
  ;; transformed in strings and consecutive non-blank characters are
  ;; grouped into a single string.
  (letrec ((build-token 
	    (lambda (l)
	      (string->symbol (list->string (reverse l)))))
	   (build-string 
	    (lambda (l)
	      (list->string (reverse l))))
	   (f (lambda (l)
		(let loop ((l l)
			   (a '())
			   (t '()))
		  (cond
		   ((null? l) ; list has ended
		    (cond
		     ((null? a) (build-token t)) ; it was the first word => token
		     ((null? t) (reverse a)) ; there were no pending chars
		     (else (reverse (cons (build-string t) a))))) ; there were pending chars
		   ((list? (car l)) ; (car l) is a sublist
		    (if (null? t)
			(loop (cdr l)
			      (cons (f (car l)) a)
			      '())
			(loop l 
			      (cons (if (null? a)
					(build-token t)
					(build-string t))
				    a) 
			      '())))
		   ((blank? (car l)) ; (car l) is blank
		    (if (null? a) 
			;; the after-token blank, which is removed;
			;; also, the token is converted to a symbol
			(loop (cdr l)
			      (cons (build-token t) a)
			      '())
			;; another blank
			(loop (cdr l)
			      (cons (car l) 
				    (if (null? t) a (cons (build-string t) a)))
			      '())))
		   (else ; (car l) is a character, so it is put in t
		    (loop (cdr l)
			  a
			  (cons (car l) t))))))))
    (cdr (f (append (list #\b #\o #\d #\y #\space) sl)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (recognize-delimiters l)
  ;; this procedure consumes a charlist and returns a list obtained by
  ;; the substitutions
  ;;
  ;; #\[ -> 'open-delimiter
  ;; #\] -> 'closed-delimiter
  ;;
  ;; which are not applied to two or more consecutive equal characters
  ;; (in order to allow escaping sequences for those characters)
  (let loop ((l l)
	     (a '()))
    (cond
     ((null? l)
      (reverse a))
     ((delimiter? (car l))
      (if (and (not (null? (cdr l)))
	       (equal? (car l) (cadr l)))
	  ;; two consecutive and equal delimiters are not a delimiter,
	  ;; but they form a single character.
	  (loop (cddr l)
		(cons (car l) a))
	  ;; otherwise, we really have a delimiter
	  (loop (cdr l)
		(cons (if (open-delimiter? (car l))
			  'open-delimiter
			  'closed-delimiter)
		      a))))
     (else
      (loop (cdr l)
	    (cons (car l) a))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (tokens->schtml.struct l)
  ;; this procedure consumes a structured list and translates each
  ;; sublist to the appropriate schtml construction.
  (letrec ((f (lambda (x)
		;(deb "x" x)
		(if (list? x)
		    (let ((s (car x)))
		      ;(deb "s" s)
		      (case s
			((data firma c r big big-c big-r huge huge-c huge-r dida dida-l dida-r) 
			 `(p ((class ,(symbol->string (car x)))) ,@(map f (cdr x))))
			((annuncio-c)
			 `(div ((class ,(symbol->string (car x)))) ,@(map f (cdr x))))
			((keywords) 
			 `(!-- "keywords: " ,@(cdr x)))
			((titolo) 
			 `(h1 ,@(map f (cdr x))))
			((sottotitolo) 
			 `(h2 ,@(map f (cdr x))))
			((sottosottotitolo) 
			 `(h3 ,@(map f (cdr x))))
			((link) 
			 `(a ((href ,(cadr x))) 
			     ,@(if (null? (cddr x))
				   (list (cadr x))
				   (map f (cddr x)))))
			((link-hash)
			 (let ((tgt (string-append
				     (cadr x) 
				     "#"
				     (cadddr x))))
			   `(a ((href ,tgt))
			       ,@(if (null? (cddddr x))
				     (list tgt)
				     (map f (cddddr x))))))
			((target) 
			 `(a ((name ,(cadr x))) ,@(map f (cddr x))))
			((email) 
			 `(a ((href ,(string-append "mailto:" (cadr x)))) ,(cadr x)))
			((file) 
			 `(a ((href ,(string-append 
				      ;eastweed-homepage-hostname
				      (eastweed 'get 'homepage-prefix)
				      (eastweed 'get 'files-prefix) (cadr x))))
			     ,@(map f (cddr x))))
			((image image-l image-r) ; syntax = [image FILE alternate text]
			 `(p ((class ,(symbol->string s))
			      (align ,(case s
					((image-l) "left")
					((image-r) "right")
					(else "center"))))
			     (img ((src ,(string-append 
					  (eastweed 'get 'homepage-prefix)
					  (eastweed 'get 'image-prefix) (cadr x)))
				   (alt ,(if (null? (cddr x))
					     (cadr x)
					     (apply string-append (cddr x))))
				   ))))
			((image-fl image-fr) ; syntax = [image-fl FILE alternate text]
			 `(img ((class ,(symbol->string s))
				(src ,(string-append 
				       (eastweed 'get 'homepage-prefix)
				       (eastweed 'get 'image-prefix) (cadr x)))
				(alt ,(if (null? (cddr x))
					  (cadr x)
					  (apply string-append (cddr x))))
				(align ,(case s
					  ((image-fl) "left")
					  (else "right"))))))
			((image-inline) ; syntax = [image-inline FILE alternate text]
			 `(img ((class ,(symbol->string s))
				(src ,(string-append 
				       (eastweed 'get 'homepage-prefix)
				       (eastweed 'get 'image-prefix) (cadr x)))
				(alt ,(if (null? (cddr x))
					  (cadr x)
					  (apply string-append (cddr x))))
				)))
			((image-css) ; syntax = [image-css FILE CSS alternate text]
			 `(img ((class ,(cadddr x))
				(src ,(string-append 
				       (eastweed 'get 'homepage-prefix)
				       (eastweed 'get 'image-prefix) (cadr x)))
				(alt ,(if (null? (cdddr x))
					  (cadr x)
					  (apply string-append (cdddr x))))
				)))
			((lp ln)
			 (cons (if (equal? (car x) 'lp) 'ul 'ol)
			       (map (lambda (i)
				      `(li ,@(map f i)))
				    (list-split (cdr x) #\newline))))
			((verbatim-input)
			 `(pre ,(list->string 
				 (file->charlist 
				  (string-append
				   (eastweed 'get 'verbatim-files-prefix)
				   (cadr x))))))

			;((my-tag) 
			; ;(deb "my-tag::(cadr x)" (cadr x))
			; ;(deb "my-tag::eastweed-my-tags" (eastweed 'get 'my-tags))
			; (let ((y (assoc (string->symbol (cadr x)) (eastweed 'get 'my-tags))))
			;   (if (not y)
			;       (errmsg "did not find tag " (cadr x) " in (eastweed 'get 'my-tags)")
			;       ((cdr y) (cddr x)))))

			(else
			 ;(deb "(symbol? s)" (symbol? s))
			 (let ((y (assoc s (eastweed 'get 'my-tags))))
			   ;(deb "y" y)
			   (if (not y)
			       (cons s (map f (cdr x)))
			       ((cdr y) (cddr x)))))))
		    (case x
		      ((br) `(br))
		      ((break) `(br ((clear "all"))))
		      ((hash) "#")
		      (else x))))))
    ;(deb "l" l)
    (map f l)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (txt->schtml filename . prefix)
  ;(deb "txt->schtml:filename" filename)
  (replace-newlines.struct
   (tokens->schtml.struct
    (replace-spaces.struct
     (make-paragraphs.struct
      (tokenize.struct
       (build-structure
	(recognize-delimiters
	 (compact-blanks
	  (remove-trailing-blanks
	   (reverse
	    (remove-trailing-blanks
	     (remove-comments-and-reverse
	      (file->charlist filename))))))))))))))

;; NOTATION: proc.struct = structured version of the procedure proc,
;; that is, a version which recursively applies itself to all the
;; sublists. TODO: need to check whether group-paragraphs,
;; tokens->schtml and wrap-paragraphs are already structured or not.
