; Eastweed - EAsy STatic WEb EDitor
; Copyright (C) 2006, 2007, 2008, 2009 Gianni Ciolli <gc@inatna.it> 
; file module-update.scm, version 0.7beta, date 2009-02-26
;
; Based on existing code (C) 2001, 2004 by Marco Maggesi
; <maggesi@math.unifi.it> and Gianni Ciolli <ciolli@math.unifi.it>
;
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

;; module-update.scm - this is the Eastweed module to update the local
;; instance of the website.

;;; TODO: aggiungere a create-filelist.recursive un meccanismo di
;;; escape per alcune directory.

(eastweed 'require 'utils)
(eastweed 'require 'schtml)
(eastweed 'require 'text2schtml)
(eastweed 'require 'nonportable)

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (txt-file? p)
  (let* ((x (x-path->string p))
	 (l (string-length x)))
    (and (> l 4)
	 (equal? (substring x (- l 4) l)
		 ".txt"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (schtml-file? p)
  (let* ((x (x-path->string p))
	 (l (string-length x)))
    (and (> l 6)
	 (equal? (substring x (- l 4) l) ".scm")
	 (not (equal? (substring x 0 2) "x-"))
	 (not (equal? (substring x 0 2) "z-")))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (change-filename-extension s e)
  (let ((l (string-length s)))
    (if (> l 4)
	(string-append (substring s 0 (- l 4)) e)
	#f)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (remove-filename-extension s)
  (let ((l (string-length s)))
    (if (> l 4)
	(substring s 0 (- l 4))
	(errmsg "remove-filename-extension: string " s " too small."))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (remove-trailing-substring s s0)
  (let ((n (string-length s0)))
    (if (> n (string-length s))
	s
	(if (string=? (substring s 0 n) s0)
	    (substring s n)
	    s))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (string-reverse s)
  (list->string (reverse (string->list s))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (remove-tail-substring s s0)
  (string-reverse
   (remove-trailing-substring (string-reverse s)
			      (string-reverse s0))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (remove-trailing-./-from-path p)
  (remove-trailing-substring (x-path->string p) "./"))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (create-filelist adir pred)
  ;; dir must be a valid argument for x-build-path
  (letrec
      ((f (lambda (rdir a) ; d=relative dir, a=already found files
	    (let* ((l (x-ls (x-build-path adir rdir)))
		   (lf (filter pred l))
		   (ld (filter 
			(lambda (p)
			  (directory-exists? 
			   (build-path adir rdir p)))
			l)))
	      (let loop ((ld ld)
			 (a (append a 
				    (map
				     (lambda (x)
				       (build-path rdir x))
				     lf))))
		(if (null? ld)
		    a
		    (loop (cdr ld)
			  (f (build-path rdir (car ld))
			     a)))))))
       (filelist (f (x-string->path "./") '())))
    filelist))

(define file-path cdr)
(define file-type car)

(define (file->path f)
  (case (file-type f)
    ((txt db schtml) (build-path (x-string->path (eastweed 'get 'data-prefix)) (file-path f)))
    (else (errmsg "unknown file type in file " f))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (filelist-recents)
  (let ((t (x-file-time 
	    (eastweed 'get 'update-timestamp-file))))
    (filter
     (lambda (f)
       (> (x-file-time (file->path f)) t))
     (filelist-all))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (build-schtml-file f)
  (let-values
   (((base name mbd?) (split-path (file-path f))))

   (let* ;((sect (cadr f))
       ((html-filename (string-append (eastweed 'get 'web-pages-prefix)
				      (x-path->string base)
				      (change-filename-extension 
				       (x-path->string name)
				       ".html")))
	(schtml-filename (string-append (eastweed 'get 'data-prefix)
					(x-path->string base)
					(x-path->string name))))
     ;; 1) build page
     (main-build (vector schtml-filename html-filename))
     ;; 2) tidy it
     (x-system (string-append (eastweed 'get 'html-tidy-string)
			      html-filename)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (estrai-titolo x) 
  ;; (p t1 ... tN) oppure (p ((class "x") (...)) t1 ... tN)
  (apply string-append (cdar x)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (estrai-keywords x) 
  (let ((p (assoc '!-- x)))
    (if (and (pair? p) 
	     (pair? (cdr p))
	     (equal? "keywords: " (cadr p)))
	(cddr p)
	#f)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (string-last-index s c)
  (let loop ((l (reverse (string->list s)))
	     (n (string-length s)))
    (cond
     ((null? l) #f)
     ((equal? (car l) c) 
      n)
     (else
      (loop (cdr l)
	    (- n 1))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (filename-split f)
  ;; Structure of a filename: BASE.LANG.SUFFIX
  ;; where LANG is the car of an element of (eastweed 'get 'languages)
  ;; and SUFFIX is one of "scm" or "txt"
  (let* ((n (string-length f))
	 (n2 (string-last-index f #\.))
	 (n1 (string-last-index (substring f 0 (- n2 1)) #\.)))
    (values
     (substring f  0 (- n1 1))
     (substring f n1 (- n2 1))
     (substring f n2 n))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (filename-lang f)
  (let-values (((basename lang ext)
		(filename-split f)))
	      lang))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (filename-default-lang? f)
  (equal?
   (filename-lang f)
   (symbol->string (caar (eastweed 'get 'languages)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (build-txt-file f . opt) ; optional argument: stile. Default is stile-txt
  (let-values
   (((base name mbd?) (split-path (file-path f)))
    ((basename lang ext)
     (filename-split (path->string (file-path f)))))
   (let* ;((sect (cadr f))
       ((html-filename (string-append 
			(eastweed 'get 'web-pages-prefix)
			basename
			(if (equal? lang (symbol->string 
					  (caar (eastweed 'get 'languages))))
			    ""
			    (string-append
			     "."
			     lang))
			".html"))
	(txt-filename (string-append (eastweed 'get 'data-prefix)
				     basename
				     "."
				     lang 
				     "."
				     ext))
	(schtml (txt->schtml txt-filename))
	;(titolo (apply string-append (estrai-titolo (car schtml))))
	(titolo (estrai-titolo schtml))
	(corpo (cdr schtml)))
     ;; 1) build page
     (let ((stile 
	    ((eval 
	      (eastweed 'get (if (null? opt) 'stile-txt (car opt))))
	     basename lang "txt")))
       (schtml->html ((stile titolo
			     (change-filename-extension
			      (remove-trailing-./-from-path 
			       (file-path f))
			      ".html"))
		      corpo)
		     html-filename))
     ;; 2) tidy it
     (x-system (string-append (eastweed 'get 'html-tidy-string) 
			      html-filename))
     )))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (update-files filelist)
  (map (lambda (f)
	 (disp-n "File " 
		 (file-path f)
		 " , "
		 (case (file-type f) 
		   ((txt)    "file di testo")
		   ((db)     "file in archivio")
		   ((schtml) "pagina schtml"))
		 " ... ")
	 (case (car f)
	   ((txt)    (build-txt-file f))
	   ((db)     (build-txt-file f stile-db-item))
	   ((schtml) (build-schtml-file f))
	   (else (disp "Cannot build file " f)))
	 (disp "aggiornato."))
       filelist)
  (disp "Aggiornati " (length filelist) " files.")
  (x-system 
   (string-append "touch " 
		  (eastweed 'get 'update-timestamp-file))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (update-recent-files)
  (let ((l (filelist-recents)))
    ;;(deb "update-recent-files:l" l)
    (if (null? l)
	(disp "Nessun file è cambiato dall'ultimo aggiornamento.")
	(begin
	  (disp "Aggiorno i files recenti.")
	  (update-files l)
	  (disp "Aggiornamento compiuto.")))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (update-all-files)
  (disp "Aggiorno tutti i files, recenti o no.")
  (update-files (filelist-all))
  (disp "Aggiornamento compiuto."))


;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (db-path? p)
  ;;(deb "db-path?:p" p)
  (string=? (eastweed 'get 'db-prefix)
	    (substring 
	     (remove-trailing-./-from-path p)
	     0 (string-length (eastweed 'get 'db-prefix)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
; filelist is a list of filenames, i.e. pairs (type . path) where path
; is the path of the source file and type is one of ('schtml 'txt).
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (filelist-all)
  (map (lambda (p)
	 (cons 
	  (cond
	   ((db-path? p) 'db)
	   ((txt-file? p) 'txt)
	   ((schtml-file? p) 'schtml))
	  p))
       (append
	(create-filelist (eastweed 'get 'data-prefix) txt-file?)
	(create-filelist (eastweed 'get 'data-prefix) schtml-file?))))

(define (filelist-db)
  (map (lambda (p)
	 (cons 'txt p))
       (create-filelist (string-append 
			 (eastweed 'get 'data-prefix)
			 (eastweed 'get 'db-prefix))
			txt-file?)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (header.filename h)
  (vector-ref h 0))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (header.date h)
  (vector-ref h 1))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (header.titolo h)
  (vector-ref h 2))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (header.keywords h)
  (vector-ref h 3))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(require (lib "list.ss"))
(define (update-db)
  ;; build l, the list of the summaries of the articles in the
  ;; database, sorted and split in fixed-size chunks, one for each db
  ;; index page.
  (display "Costruisco gli indici dell'archivio...")
  (let* ((l
	  (list-split-fixed-size 
	   (quicksort
	    (map (lambda (p)
		   (let* ((txt-filename (remove-trailing-./-from-path (cdr p)))
			  (date (string->number (substring txt-filename 0 8)))
			  (txt-path (string-append (eastweed 'get 'data-prefix)
						   (eastweed 'get 'db-prefix)
						   txt-filename))
			  (schtml (txt->schtml txt-path)))
		     (vector txt-filename 
			     date 
			     (estrai-titolo schtml)
			     (estrai-keywords schtml)
			     )))
		 (filelist-db))
	    (lambda (x y)
	      (string<? (header.filename x)
			(header.filename y))))
	   eastweed-index-chunksize))
	 (m (length l))) ; number of index pages
    (let loop ((i 1)
	       (l l))
      (if (null? l)
	  (disp " fatto.")
	  (let* ((index-filename (string-append 
				  eastweed-db-name 
				  "-"
				  (number->string i) ".html"))
		 (index-url (string-append
				  (eastweed 'get 'web-pages-prefix)
				  index-filename)))
	    ;; 1) build page
	    (schtml->html 
	     ((stile-db (string->symbol eastweed-db-name)
			index-filename
			i
			m
			(car l))
	      '())
	     index-url)
	    ;; 2) tidy it
	    (x-system (string-append (eastweed 'get 'html-tidy-string)
				     index-url))
	    (loop (+ i 1)
		  (cdr l)))))))
