; Eastweed - EAsy STatic WEb EDitor
; Copyright (C) 2006, 2007, 2008, 2009 Gianni Ciolli <gc@inatna.it> 
; file module-utils.scm, version 0.7beta, date 2009-02-26
;
; Based on existing code (C) 2001, 2004 by Marco Maggesi
; <maggesi@math.unifi.it> and Gianni Ciolli <ciolli@math.unifi.it>
;
; This program is free software; you can redistribute it and/or modify
; it under the terms of the GNU General Public License as published by
; the Free Software Foundation; either version 2 of the License, or
; (at your option) any later version.
;
; This program is distributed in the hope that it will be useful,
; but WITHOUT ANY WARRANTY; without even the implied warranty of
; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
; GNU General Public License for more details.
;
; You should have received a copy of the GNU General Public License
; along with this program; if not, write to the Free Software
; Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

;; module-utils.scm - this is the Eastweed utilities module.

(define (url-escape-char c)
  (case c
    ((#\space) "%20")
    ((#\è) "%E8")
    ((#\é) "%E9")
    ((#\°) "%B0")
    (else (string c))))

(define (url-escape-string s)
  (let loop ((l (string->list s))
	     (a ""))
    (if (null? l)
	a
	(loop (cdr l)
	      (string-append a 
			     (url-escape-char (car l)))))))

(define (link-file prefisso file)
  `(p "file " (a ((href ,(url-escape-string 
			  (string-append prefisso file))))
		 (tt ,file))))

(define (thumbnail-page titolo-breve 
			titolo-esteso 
			scm-prefix
			html-base-url
			img-base-url
			img-suffix
			tn-prefix
			x y
			prev
			this
			next
			prev-img
			this-img
			next-img
			up) ;; u = url up
  (with-output-to-file (string-append scm-prefix 
				      (number->string this)
				      ".scm")
    (lambda ()
      (display ",")
      (write `(stile-thumbnail-page
	       ,titolo-breve
	       ,titolo-esteso
	       ,html-base-url
	       ,img-base-url
	       ,img-suffix
	       ,tn-prefix
	       ,prev
	       ,this
	       ,next
	       ,prev-img
	       ,this-img
	       ,next-img
	       ,up)))))

(define get-x car)
(define get-y cadr)
(define get-nome caddr)

(define (list->thumbnails html-base-url
			  img-base-url 
			  tn-prefix 
			  l0)
  (let loop ((a '())
	     (l l0)
	     (i 0))
    (if (null? l)
	a
	(loop (append 
	       a
	       (let ((nome (caddr (car l))))
		 (list " "
		       `(a ((href ,(url-escape-string 
				    (string-append
				     html-base-url
				     (number->string i)
				     ".html"))))
			   (img ((src ,(url-escape-string
					(string-append
					 img-base-url
					 tn-prefix
					 nome)))
				 (alt ,nome)))))))
	      (cdr l)
	      (+ i 1)))))

(define (thumbnail-images titolo-breve
			  titolo-esteso
			  scm-prefix
			  up-url
			  html-base-url
			  img-base-url
			  img-suffix
			  tn-prefix
			  l) ;; l = lista (x y n)
  (if (file-exists? (string-append scm-prefix
				   "0.scm"))
      (list->thumbnails html-base-url 
			img-base-url 
			tn-prefix 
			l)
      (let ((n (length l))
	    (v (list->vector l)))
	(let loop ((i 0))
	  (if (= i n)
	      (list->thumbnails html-base-url 
				img-base-url 
				tn-prefix 
				l)
	      (let ((prev (modulo (+ i n -1) n))
		    (this i)
		    (next (modulo (+ i 1) n)))
		(thumbnail-page titolo-breve
				titolo-esteso
				scm-prefix
				html-base-url
				img-base-url
				img-suffix
				tn-prefix
				(get-x (vector-ref v this))
				(get-y (vector-ref v this))
				prev
				this
				next
				(get-nome (vector-ref v prev))
				(get-nome (vector-ref v this))
				(get-nome (vector-ref v next))
				up-url)
		(loop (+ i 1))))))))


	  
;;  `(a ((href ,(url-escape-string f)))
;;      (img ((src ,(url-escape-string t))
;;	    (alt ,(url-escape-string f))
;;	    ))
;;      " "))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define disp
  (lambda x
    (map display x)
    (newline)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define disp-n
  (lambda x
    (map display x))); no newline!

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define disp-s
  (lambda x
    (map (lambda (t) 
	   (display t)
	   (display " "))
	 x)
    (newline)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; TODO: how does this relate to the existing error procedure?
(define errmsg ;; error procedure
  (lambda x
    (newline)
    (display "ERROR: ")
    (map (lambda (t) 
	   (display t)
	   (display " "))
	 x)
    (newline)
    (newline)
    (exit))) ;; certainly an error!

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define logmsg ;; logging procedure
  (lambda x
    (display "LOG: ")
    (map (lambda (t) 
	   (display t)
	   (display " "))
	 x)
    (newline)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (deb s x) ;; procedure useful in debugging
  (let ((stderr (current-output-port)))
    (display "DEBUG <" stderr)
    (display s         stderr)
    (display ">: <"    stderr)
    (display x         stderr)
    (display ">"       stderr)
    (newline           stderr)
    x))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (intersperse x l0)
  (if (null? l0) 
      '()
      (let loop ((a (list (car l0)))
		 (l (cdr l0)))
	(if (null? l)
	    (reverse a)
	    (loop (cons (car l)
			(cons x a))
		  (cdr l))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define tavola
  (lambda (x0 y0 x y n b l)
    `(p ((align "center"))
	(img ((src    ,(string-append n 
				      "-" 
				      (number->string x)
				      "x"
				      (number->string y)
				      ".jpg"))
	      ;(width  ,(number->string x))
	      ;(height ,(number->string y))
	      (alt    ,n)
	      (usemap ,(string-append "#" n))))
	(map ((name ,n))
	     ,@(map (lambda (z)
		      `(area ((shape "poly")
			      (coords ,(apply
					string-append
					(intersperse "," (map number->string (cdr z)))))
			      (href ,(string-append (car z) ".html"))
			      (alt ,(car z)))))
		    (if b
			(append l (list (list b 0 0 0 10000 10000 10000 10000 0)))
			l))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (string-split s c) ; splits a string in a pair of two
			   ; substrings
  (let loop ((a '())
	     (l (string->list s)))
    (if (null? l)
	(cons "" s)
	(if (equal? (car l) c)
	    (cons (list->string (reverse (cons (car l) a)))
		  (list->string (cdr l)))
	    (loop (cons (car l) a) (cdr l))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (basename-append s1 s2)
  (let* ((x (string-split s2 #\.))
	 (p (car x))
	 (s (cdr x)))
    (string-append p s1 s)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (basename-prepend s1 s2)
  (let* ((x (string-split s2 #\/))
	 (p (car x))
	 (s (cdr x)))
    (string-append p s1 s)))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (split l0 x)
  (let loop ((a '())
	     (l l0))
    (if (null? l)
	(values l0 #f)
	(if (equal? (car l) x)
	    (values (reverse a) l)
	    (loop (cons (car l) a) (cdr l))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (list-head l k)
  (let loop ((l l)
	     (a '())
	     (k k))
    (if (or (null? l)
	    (zero? k))
	(reverse a)
	(loop (cdr l)
	      (cons (car l) a)
	      (- k 1)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (delete-duplicates s) ;; da t-y-scheme
  (if (null? s) s
      (let ((a (car s)) (d (cdr s)))
	(if (memv a d) (delete-duplicates d)
	    (cons a (delete-duplicates d))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (file.html file.txt)
  (string-append 
   (substring file.txt 
	      0 
	      (- (string-length file.txt) 4))
   ".html"))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (file.html-untidied file.txt)
  (string-append
   (substring file.txt 
	      0 
	      (- (string-length file.txt) 4))
   "-untidied.html"))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (file.scm file.txt)
  (string-append
   (substring file.txt 
	      0 
	      (- (string-length file.txt) 4))
   ".scm"))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;(define (file.htm file.txt)
;  (string-append 
;   (substring file.txt 
;	      0 
;	      (- (string-length file.txt) 4))
;   ".htm"))
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;(define (file.pdf file.txt)
;  (string-append 
;   (substring file.txt 
;	      0 
;	      (- (string-length file.txt) 4))
;   ".pdf"))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (filter f l)
  (let loop ((l l)
	     (a (list)))
    (if (null? l)
	(reverse a)
	(loop (cdr l)
	      (if (f (car l))
		  (cons (car l) a)
		  a)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (filter-out-false l)
  (filter (lambda (x)
	    x)
	  l))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (numera-lista l)
  (let loop ((l l)
	     (a '())
	     (n 0))
    (if (null? l)
	(reverse a)
	(loop (cdr l)
	      (cons (cons n (car l)) a)
	      (+ n 1)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (list-split l sep)
  (let loop ((l l)
	     (a '())
	     (p '()))
    (if (null? l)
	(reverse (cons (reverse p) a))
	(if (equal? (car l) sep)
	    (loop (cdr l)
		  (cons (reverse p) a)
		  '())
	    (loop (cdr l)
		  a
		  (cons (car l) p))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (list-split-fixed-size l size)
  (let loop ((l l)
	     (a '())
	     (p '())
	     (i 0))
    (if (null? l)
	(reverse (cons (reverse p) a))
	(if (equal? i size)
	    (loop l
		  (cons (reverse p) a)
		  '()
		  0)
	    (loop (cdr l)
		  a
		  (cons (car l) p)
		  (+ 1 i))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(require (lib "string.ss"))
(define (active-url url)
  (deb "url" url)
  (let ((f (lambda (x y)
	     (if (string=? x y)
		 `((a ((href ,x))
		      ,y))
		 `("/" 
		   (a ((href ,x))
		      ,y)))))
	(l0 (regexp-split "/" url)))
    (deb "l0" l0)
    (deb "(null? l0)" (null? l0))
    (if (null? l0)
	`(nbsp)
	(let loop ((l l0)
		   (a '())
		   (a1 '()))
	  (newline)
;;	  (deb "a1" a1)
;;	  (deb "a" a)
;;	  (deb "l" l)
	  (if (null? l)
	      (apply append (reverse (map f a a1)))
	      (loop (cdr l)
		    (cons (apply string-append
				 (cons "/"
				       (intersperse "/"
						    (reverse (cons (car l) a1)))))
			  a)
		    (cons (car l) a1)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (url-toplevel? s)
  (let loop ((l (string->list s)))
    (cond 
     ((null? l) #t)
     ((equal? (car l) #\/) #f)
     (else (loop (cdr l))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
(define (format-date x)
  (let* ((y (quotient x 10000))
	 (m (quotient (- x (* 10000 y)) 100))
	 (d (- x (* 10000 y) (* 100 m))))
    (string-append
     (number->string d)
     "/"
     (number->string m)
     "/"
     (number->string y))))
